/******************************************************************************
*	Module: dcf_demo.c
*	Demo DCF77 Radio-Clock Application
*
* 	Copyright (C) 2004 Volker Troyke, www.troyke.de
*
* 	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include "global.h"
#include "lcd.h"
#include "ds1302.h"
#include "dcf77.h"
#include "math.h"

u08 prescale;
u08 update;

struct dt act_date;
struct dt dcf_date;

SIGNAL(SIG_OVERFLOW0)        /* signal handler for tcnt0 overflow interrupt */
{
	prescale++;
	dcf77_timetick();

	if(prescale == TICK_100M)
	{
		prescale = 0;
		update = 1;
	}
}

void display_date(u08 col, struct dt *date)
{
	char data[5];

  	lcd_gotoxy(col,2);
	if(date->d.tag < 10)
	{
		lcd_puts_P(PSTR(" "));
		char2str(date->d.tag, data,1);
		lcd_puts(data);
	}
	else
	{
		char2str(date->d.tag, data,2);
		lcd_puts(data);
	}

	lcd_puts_P(PSTR("."));
	char2str(date->d.monat, data,2);
	lcd_puts(data);
	lcd_puts_P(PSTR("."));
	char2str(date->d.jahr, data,2);
	lcd_puts(data);

	if(dcf_valid)
	{
		lcd_puts_P(PSTR(" DCF-sync   "));
	}
	else if(sync_ok)
	{
		lcd_puts_P(PSTR(" Reading DCF"));
	}
	else if(signal_ok)
	{
		lcd_puts_P(PSTR(" Signal OK  "));
	}
	else
	{
		lcd_puts_P(PSTR(" No Signal! "));
	}



	lcd_gotoxy(col,1);
	if(date->t.h < 10)
	{
		lcd_puts_P(PSTR(" "));
		char2str(date->t.h, data,1);
		lcd_puts(data);
	}
	else
	{
		char2str(date->t.h, data,2);
		lcd_puts(data);
	}

	if(signal_ok)
	{
		lcd_puts_P(PSTR(":"));
	}
	else
	{
		lcd_puts_P(PSTR("."));
	}

	char2str(date->t.m, data,2);
	lcd_puts(data);
	if(dcf_valid)
	{
		lcd_puts_P(PSTR(":"));
	}
	else
	{
		lcd_puts_P(PSTR("."));
	}
	char2str(date->t.s, data,2);
	lcd_puts(data);

	if(act_date.t.timezone & 0x01)
	{
		lcd_puts_P(PSTR(" Sommerzeit "));
	}
	else
	{
		lcd_puts_P(PSTR(" Winterzeit "));
	}

}

int main(void)
{
	u08 led;

	cbi(DDRB,PB1); 					// Contrast Pin, switch to input
	cbi(PORTB,PB1);					// Contrast Pin, switch to high impedance
	ACSR=0x80;  						// analog comparator switched off
	GIMSK=0x40;							// Enable Int 0 for DCF77 Receiver
	MCUCR=0x02;							// Int 0 falling Edge

	cbi(DDRD,PD2);						// switch DCF77 Port to input
	cbi(PORTD,PD2);					// high Impedance

	TCCR0=0x02;							// counter prescaler CK/8
	TCNT0=0x01; 						// set up timer0 -> 125 INTs/s
	TIMSK=0x02; 					// enable timer0 int

	init_ds1302();
	dcf77_init();
	lcd_init();

	prescale		= 0;
	led			= 0;
	update 		= 0;

	read_time_ds1302(&act_date);

	lcd_clear();
	lcd_mode(LCD_ON_CURSOR_OFF);

//	wdt_reset();
//	wdt_enable(0x07);

	sei();

	for(;;)    /* loop forever */
	{
		if(dcf77_update)					// new DCF-Bit received
		{
			dcf77_update = 0;
			dcf77_process_data();
		}

		if(update_time)					// DCF Time Information complete, update display Time and Timekeeper
		{
			update_time 		= 0;
			dcf77_tick 			= 0;
			act_date.t.s 		= dcf_date.t.s;
			act_date.t.m 		= dcf_date.t.m;
			act_date.t.h 		= dcf_date.t.h;
			act_date.d.tag 	= dcf_date.d.tag;
			act_date.d.wt 		= dcf_date.d.wt;
			act_date.d.monat 	= dcf_date.d.monat;
			act_date.d.jahr 	= dcf_date.d.jahr;

			write_time_ds1302(&act_date);
		}

		if(inc_time)						// Inrement Display Time, 1 second Intervall
		{
			inc_time = 0;
			increment_time(&act_date);
			display_date(1,&act_date);
		}

		if(update)							// 100ms Intervall
		{
//			wdt_reset();					// Watchdog Reset
			update = 0;

			if(led == 0)							// alive LED
			{
				cbi(PORTB,PB5);
				led = 1;
			}
			else
			{
				sbi(PORTB,PB5);
				led = 0;
			}
		}
	}
}
